-- ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ --
-- Gritter
--
-- Purpose: Removes snow (created by seasons) in its workArea
-- 
-- Authors: Timmiej93
--
-- Copyright (c) Timmiej93, 2018
-- For more information on copyright for this mod, please check the readme file on Github
--
-- ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ --

Gritter = {}

function Gritter.initSpecialization()
    WorkArea.registerAreaType("gritter")
end

function Gritter.prerequisitesPresent(specializations)
	return (SpecializationUtil.hasSpecialization(WorkArea, specializations)
		and SpecializationUtil.hasSpecialization(Fillable, specializations))
end

function Gritter:preLoad(savegame)
	self.loadWorkAreaFromXML = Utils.overwrittenFunction(self.loadWorkAreaFromXML, Gritter.loadWorkAreaFromXML)
end

function Gritter:load(saveGame)

	self.canFunction = true

	if (not g_modIsLoaded["FS17_RM_Seasons"]) then
		print("WARNING: Seasons mod isn't loaded, gritter only works on snow from the Seasons mod! (Gritter script by Timmiej93)")
		self.canFunction = false
		return
	end

	self.processGritterAreas = Gritter.processGritterAreas
	self.getHasSpray = Utils.overwrittenFunction(self.getHasSpray, Gritter.getHasSpray)
end

function Gritter:updateTick(dt)
	if (self:getIsActive() and self.canFunction) then
		local workAreas = self:getTypedNetworkAreas(WorkArea.AREATYPE_GRITTER, false)

		if (self.isServer) then
			if (table.getn(workAreas) > 0) then
				self:processGritterAreas(workAreas)
			end
            g_currentMission.missionStats:updateStats("workedTime", dt/(1000*60))
        end
    end
end

function Gritter:loadWorkAreaFromXML(superFunc, workArea, xmlFile, key)
    local retValue = true
    if (superFunc ~= nil) then
        retValue = superFunc(self, workArea, xmlFile, key)
    end

    if (workArea.type == WorkArea.AREATYPE_DEFAULT) then
        workArea.type = WorkArea.AREATYPE_GRITTER
    end

    return retValue
end

function Gritter:processGritterAreas(workAreas)

	if (self.canFunction and self:getFillLevel() > 0) then

		local fillTypes = self:getCurrentFillTypes()
		local snowFilltypeIdx = FillUtil.FILLTYPE_SNOW

		local notSnowFilled = true
		if (snowFilltypeIdx ~= nil) then
			for _,filltype in pairs(fillTypes) do
				if (filltype == snowFilltypeIdx) then
					notSnowFilled = false
					break
				end
			end
		end
		if (notSnowFilled) then
			local numAreas = table.getn(workAreas)
		    local areaSum = 0
		    for i=1, numAreas do
		        local x = workAreas[i][1]
		        local z = workAreas[i][2]
		        local x1 = workAreas[i][3]
		        local z1 = workAreas[i][4]
		        local x2 = workAreas[i][5]
		        local z2 = workAreas[i][6]

		        g_seasons.snow:removeSnow(x,z, x1,z1, x2,z2, 10)
		    end
		end
	end
end

function Gritter:getHasSpray(superFunc, fillTypes, usage)

	local speed = self:getLastSpeed()
	if (speed ~= nil) then
		usage = usage * (speed / 50)
	end

	return superFunc(self, fillTypes, usage)
end

function Gritter:delete()end
function Gritter:mouseEvent(posX, posY, isDown, isUp, button)end
function Gritter:keyEvent(unicode, sym, modifier, isDown)end
function Gritter:update(dt)	end
function Gritter:draw()end